"""Module to put any functions that are referred to in the "callables" section of FixHeaderApplyTransforms.yaml"""

import attrs
import os
from nipype.utils.filemanip import split_filename


def output_image_default(inputs):
    return _gen_filename("output_image", inputs=inputs)


def output_image_callable(output_dir, inputs, stdout, stderr):
    outputs = _list_outputs(
        output_dir=output_dir, inputs=inputs, stdout=stdout, stderr=stderr
    )
    return outputs["output_image"]


# Original source at L465 of <nipype-install>/interfaces/ants/resampling.py
def _gen_filename(name, inputs=None, stdout=None, stderr=None, output_dir=None):
    if name == "output_image":
        output = inputs.output_image
        if output is attrs.NOTHING:
            _, name, ext = split_filename(inputs.input_image)
            output = name + inputs.out_postfix + ext
        return output
    return None


# Original source at L522 of <nipype-install>/interfaces/ants/resampling.py
def _list_outputs(inputs=None, stdout=None, stderr=None, output_dir=None):
    outputs = {}
    outputs["output_image"] = os.path.abspath(
        _gen_filename(
            "output_image",
            inputs=inputs,
            stdout=stdout,
            stderr=stderr,
            output_dir=output_dir,
        )
    )
    return outputs



"""Module to put any functions that are referred to in the "callables" section of SpatialNormalizationRPT.yaml"""


def composite_transform_callable(output_dir, inputs, stdout, stderr):
    outputs = _list_outputs(
        output_dir=output_dir, inputs=inputs, stdout=stdout, stderr=stderr
    )
    return outputs["composite_transform"]


def elapsed_time_callable(output_dir, inputs, stdout, stderr):
    outputs = _list_outputs(
        output_dir=output_dir, inputs=inputs, stdout=stdout, stderr=stderr
    )
    return outputs["elapsed_time"]


def forward_invert_flags_callable(output_dir, inputs, stdout, stderr):
    outputs = _list_outputs(
        output_dir=output_dir, inputs=inputs, stdout=stdout, stderr=stderr
    )
    return outputs["forward_invert_flags"]


def forward_transforms_callable(output_dir, inputs, stdout, stderr):
    outputs = _list_outputs(
        output_dir=output_dir, inputs=inputs, stdout=stdout, stderr=stderr
    )
    return outputs["forward_transforms"]


def inverse_composite_transform_callable(output_dir, inputs, stdout, stderr):
    outputs = _list_outputs(
        output_dir=output_dir, inputs=inputs, stdout=stdout, stderr=stderr
    )
    return outputs["inverse_composite_transform"]


def inverse_warped_image_callable(output_dir, inputs, stdout, stderr):
    outputs = _list_outputs(
        output_dir=output_dir, inputs=inputs, stdout=stdout, stderr=stderr
    )
    return outputs["inverse_warped_image"]


def metric_value_callable(output_dir, inputs, stdout, stderr):
    outputs = _list_outputs(
        output_dir=output_dir, inputs=inputs, stdout=stdout, stderr=stderr
    )
    return outputs["metric_value"]


def out_report_callable(output_dir, inputs, stdout, stderr):
    outputs = _list_outputs(
        output_dir=output_dir, inputs=inputs, stdout=stdout, stderr=stderr
    )
    return outputs["out_report"]


def reference_image_callable(output_dir, inputs, stdout, stderr):
    outputs = _list_outputs(
        output_dir=output_dir, inputs=inputs, stdout=stdout, stderr=stderr
    )
    return outputs["reference_image"]


def reverse_forward_invert_flags_callable(output_dir, inputs, stdout, stderr):
    outputs = _list_outputs(
        output_dir=output_dir, inputs=inputs, stdout=stdout, stderr=stderr
    )
    return outputs["reverse_forward_invert_flags"]


def reverse_forward_transforms_callable(output_dir, inputs, stdout, stderr):
    outputs = _list_outputs(
        output_dir=output_dir, inputs=inputs, stdout=stdout, stderr=stderr
    )
    return outputs["reverse_forward_transforms"]


def reverse_invert_flags_callable(output_dir, inputs, stdout, stderr):
    outputs = _list_outputs(
        output_dir=output_dir, inputs=inputs, stdout=stdout, stderr=stderr
    )
    return outputs["reverse_invert_flags"]


def reverse_transforms_callable(output_dir, inputs, stdout, stderr):
    outputs = _list_outputs(
        output_dir=output_dir, inputs=inputs, stdout=stdout, stderr=stderr
    )
    return outputs["reverse_transforms"]


def save_state_callable(output_dir, inputs, stdout, stderr):
    outputs = _list_outputs(
        output_dir=output_dir, inputs=inputs, stdout=stdout, stderr=stderr
    )
    return outputs["save_state"]


def warped_image_callable(output_dir, inputs, stdout, stderr):
    outputs = _list_outputs(
        output_dir=output_dir, inputs=inputs, stdout=stdout, stderr=stderr
    )
    return outputs["warped_image"]


# Original source at L419 of <nipype-install>/interfaces/base/core.py
def BaseInterface___list_outputs(
    inputs=None, stdout=None, stderr=None, output_dir=None
):
    """List the expected outputs"""
    if True:
        raise NotImplementedError
    else:
        return None


# Original source at L54 of <nipype-install>/interfaces/mixins/reporting.py
def ReportCapableInterface___list_outputs(
    inputs=None, stdout=None, stderr=None, output_dir=None
):
    try:
        outputs = BaseInterface___list_outputs()
    except NotImplementedError:
        outputs = {}
    if _out_report is not None:
        outputs["out_report"] = _out_report
    return outputs


# Original source at L54 of <nipype-install>/interfaces/mixins/reporting.py
def _list_outputs(inputs=None, stdout=None, stderr=None, output_dir=None):
    try:
        outputs = BaseInterface___list_outputs()
    except NotImplementedError:
        outputs = {}
    if _out_report is not None:
        outputs["out_report"] = _out_report
    return outputs


# Original source at L54 of <nipype-install>/interfaces/mixins/reporting.py
def niworkflows_interfaces_reportlets__RegistrationRC___list_outputs(
    inputs=None, stdout=None, stderr=None, output_dir=None
):
    try:
        outputs = BaseInterface___list_outputs()
    except NotImplementedError:
        outputs = {}
    if _out_report is not None:
        outputs["out_report"] = _out_report
    return outputs



#!/usr/bin/env python3
from pathlib import Path
import inspect
from importlib import import_module
import click
from looseversion import LooseVersion
from pydra.engine.core import TaskBase


PKG_DIR = Path(__file__).parent.parent
TASKS_DIR = PKG_DIR / "pydra" / "tasks" / "ants"
VERSION_GRANULARITY = (
    2  # Number of version parts to include: 1 - major, 2 - minor, 3 - micro
)


@click.command(
    help="""Increment the latest version or create a new sub-package for interfaces for
a new release of AFNI depending on whether one already exists or not.

NEW_VERSION the version of AFNI to create a new sub-package for
"""
)
@click.argument("new_version", type=LooseVersion)
def increment_tool_version(new_version: LooseVersion):

    # Get the name of the sub-package, e.g. "v2_5"
    new_subpkg_name = "_".join(str(p) for p in new_version.version[:VERSION_GRANULARITY])  # type: ignore
    if not new_subpkg_name.startswith("v"):
        new_subpkg_name = "v" + new_subpkg_name
    sub_pkg_dir = TASKS_DIR / new_subpkg_name
    if not sub_pkg_dir.exists():

        prev_version = sorted(
            (
                p.name
                for p in TASKS_DIR.iterdir()
                if p.is_dir() and p.name.startswith("v")
            ),
            key=lambda x: LooseVersion(".".join(x.split("_"))).version,
        )[-1]
        prev_ver_mod = import_module(f"pydra.tasks.ants.{prev_version}")

        mod_attrs = [getattr(prev_ver_mod, a) for a in dir(prev_ver_mod)]
        task_classes = [
            a for a in mod_attrs if inspect.isclass(a) and issubclass(a, TaskBase)
        ]

        code_str = (
            f"from pydra.tasks.ants import {prev_version}\n"
            "from . import _tool_version\n"
        )

        for task_cls in task_classes:
            code_str += (
                f"\n\nclass {task_cls.__name__}({prev_version}.{task_cls.__name__}):\n"
                "    TOOL_VERSION = _tool_version.TOOL_VERSION\n"
            )

        sub_pkg_dir.mkdir(exist_ok=True)
        with open(sub_pkg_dir / "__init__.py", "w") as f:
            f.write(code_str)

    with open(sub_pkg_dir / "_tool_version.py", "w") as f:
        f.write(f'TOOL_VERSION = "{new_version}"\n')


if __name__ == "__main__":
    increment_tool_version()



#!/usr/bin/env python3
import sys
import os
import re
import fnmatch
import functools
from pathlib import Path

PACKAGE_ROOT = Path(__file__).absolute().parent.parent


@functools.lru_cache()
def load_gitignore(repo):
    gitignore = repo / ".gitignore"
    ignore = [fnmatch.translate(".git/"), fnmatch.translate(Path(__file__).name)]
    if gitignore.exists():
        ignore.extend(
            fnmatch.translate(line.strip())
            for line in gitignore.read_text().splitlines()
            if line.strip() and not line[0] == "#"
        )
    return re.compile("|".join(ignore))


cmd, new_name, *_ = sys.argv

for root_, dirs, files in os.walk(PACKAGE_ROOT):
    ignore = load_gitignore(PACKAGE_ROOT).search
    for d in [d for d in dirs if ignore(f"{d}/")]:
        dirs.remove(d)
    for f in [f for f in files if ignore(f)]:
        files.remove(f)

    root = Path(root_)
    for src in list(dirs):
        if "niworkflows" in src:
            dst = src.replace("niworkflows", new_name)
            print(f"Renaming: {root / src} ->  {root / dst}")
            os.rename(root / src, root / dst)
            dirs.remove(src)
            dirs.append(dst)
    for fname in files:
        text = Path.read_text(root / fname)
        if "niworkflows" in text:
            print(f"Rewriting: {root / fname}")
            Path.write_text(root / fname, text.replace("niworkflows", new_name))



#!/usr/bin/env python3
from pathlib import Path
import json
import yaml
import click


@click.command
@click.argument(
    "out_json_path",
    type=click.Path(path_type=Path),
    help="The output path to save the report",
)
def report_progress(out_json_path: Path):

    out_json_path.parent.mkdir(exist_ok=True, parents=True)

    SPECS_DIR = Path(__file__).parent / "nipype-auto-conv" / "specs"

    report = {}

    for spec_path in SPECS_DIR.glob("*.yaml"):
        with open(spec_path) as f:
            spec = yaml.load(f, Loader=yaml.SafeLoader)

        report[spec["task_name"]] = {
            n: not s["xfail"] for n, s in spec["tests"].items()
        }

    with open(out_json_path, "w") as f:
        json.dump(report, f)



"""Module to put any functions that are referred to in the "callables" section of FixN4BiasFieldCorrection.yaml"""

import os
from nipype import logging
from nipype.utils.filemanip import split_filename


def bias_image_callable(output_dir, inputs, stdout, stderr):
    outputs = _list_outputs(
        output_dir=output_dir, inputs=inputs, stdout=stdout, stderr=stderr
    )
    return outputs["bias_image"]


def negative_values_callable(output_dir, inputs, stdout, stderr):
    outputs = _list_outputs(
        output_dir=output_dir, inputs=inputs, stdout=stdout, stderr=stderr
    )
    return outputs["negative_values"]


def output_image_callable(output_dir, inputs, stdout, stderr):
    outputs = _list_outputs(
        output_dir=output_dir, inputs=inputs, stdout=stdout, stderr=stderr
    )
    return outputs["output_image"]


iflogger = logging.getLogger("nipype.interface")


# Original source at L891 of <nipype-install>/interfaces/base/core.py
def ANTSCommand___list_outputs(inputs=None, stdout=None, stderr=None, output_dir=None):
    metadata = dict(name_source=lambda t: t is not None)
    traits = inputs.traits(**metadata)
    if traits:
        outputs = {}
        for name, trait_spec in list(traits.items()):
            out_name = name
            if trait_spec.output_name is not None:
                out_name = trait_spec.output_name
            fname = _filename_from_source(
                name, inputs=inputs, stdout=stdout, stderr=stderr, output_dir=output_dir
            )
            if fname is not attrs.NOTHING:
                outputs[out_name] = os.path.abspath(fname)
        return outputs


# Original source at L540 of <nipype-install>/interfaces/ants/segmentation.py
def N4BiasFieldCorrection___list_outputs(
    inputs=None, stdout=None, stderr=None, output_dir=None
):
    outputs = ANTSCommand___list_outputs()
    if _out_bias_file:
        outputs["bias_image"] = os.path.abspath(_out_bias_file)
    return outputs


# Original source at L809 of <nipype-install>/interfaces/base/core.py
def _filename_from_source(
    name, chain=None, inputs=None, stdout=None, stderr=None, output_dir=None
):
    if chain is None:
        chain = []

    trait_spec = inputs.trait(name)
    retval = getattr(inputs, name)
    source_ext = None
    if (retval is attrs.NOTHING) or "%s" in retval:
        if not trait_spec.name_source:
            return retval

        # Do not generate filename when excluded by other inputs
        if any(
            (getattr(inputs, field) is not attrs.NOTHING)
            for field in trait_spec.xor or ()
        ):
            return retval

        # Do not generate filename when required fields are missing
        if not all(
            (getattr(inputs, field) is not attrs.NOTHING)
            for field in trait_spec.requires or ()
        ):
            return retval

        if (retval is not attrs.NOTHING) and "%s" in retval:
            name_template = retval
        else:
            name_template = trait_spec.name_template
        if not name_template:
            name_template = "%s_generated"

        ns = trait_spec.name_source
        while isinstance(ns, (list, tuple)):
            if len(ns) > 1:
                iflogger.warning("Only one name_source per trait is allowed")
            ns = ns[0]

        if not isinstance(ns, (str, bytes)):
            raise ValueError(
                "name_source of '{}' trait should be an input trait "
                "name, but a type {} object was found".format(name, type(ns))
            )

        if getattr(inputs, ns) is not attrs.NOTHING:
            name_source = ns
            source = getattr(inputs, name_source)
            while isinstance(source, list):
                source = source[0]

            # special treatment for files
            try:
                _, base, source_ext = split_filename(source)
            except (AttributeError, TypeError):
                base = source
        else:
            if name in chain:
                raise NipypeInterfaceError("Mutually pointing name_sources")

            chain.append(name)
            base = _filename_from_source(
                ns,
                chain,
                inputs=inputs,
                stdout=stdout,
                stderr=stderr,
                output_dir=output_dir,
            )
            if base is not attrs.NOTHING:
                _, _, source_ext = split_filename(base)
            else:
                # Do not generate filename when required fields are missing
                return retval

        chain = None
        retval = name_template % base
        _, _, ext = split_filename(retval)
        if trait_spec.keep_extension and (ext or source_ext):
            if (ext is None or not ext) and source_ext:
                retval = retval + source_ext
        else:
            retval = _overload_extension(
                retval,
                name,
                inputs=inputs,
                stdout=stdout,
                stderr=stderr,
                output_dir=output_dir,
            )
    return retval


# Original source at L885 of <nipype-install>/interfaces/base/core.py
def _gen_filename(name, inputs=None, stdout=None, stderr=None, output_dir=None):
    raise NotImplementedError


# Original source at L171 of <niworkflows-install>/interfaces/fixes.py
def _list_outputs(inputs=None, stdout=None, stderr=None, output_dir=None):
    outputs = ANTSCommand___list_outputs()
    outputs["negative_values"] = _negative_values
    return outputs


# Original source at L888 of <nipype-install>/interfaces/base/core.py
def _overload_extension(
    value, name=None, inputs=None, stdout=None, stderr=None, output_dir=None
):
    return value


# Original source at L58 of <nipype-install>/utils/filemanip.py
def split_filename(fname):
    """Split a filename into parts: path, base filename and extension.

    Parameters
    ----------
    fname : str
        file or path name

    Returns
    -------
    pth : str
        base path from fname
    fname : str
        filename from fname, without extension
    ext : str
        file extension from fname

    Examples
    --------
    >>> from nipype.utils.filemanip import split_filename
    >>> pth, fname, ext = split_filename('/home/data/subject.nii.gz')
    >>> pth
    '/home/data'

    >>> fname
    'subject'

    >>> ext
    '.nii.gz'

    """

    special_extensions = [".nii.gz", ".tar.gz", ".niml.dset"]

    pth = op.dirname(fname)
    fname = op.basename(fname)

    ext = None
    for special_ext in special_extensions:
        ext_len = len(special_ext)
        if (len(fname) > ext_len) and (fname[-ext_len:].lower() == special_ext.lower()):
            ext = fname[-ext_len:]
            fname = fname[:-ext_len]
            break
    if not ext:
        fname, ext = op.splitext(fname)

    return pth, fname, ext



"""Module to put any functions that are referred to in the "callables" section of ReadSidecarJSON.yaml"""


def acquisition_callable(output_dir, inputs, stdout, stderr):
    outputs = _list_outputs(
        output_dir=output_dir, inputs=inputs, stdout=stdout, stderr=stderr
    )
    return outputs["acquisition"]


def out_dict_callable(output_dir, inputs, stdout, stderr):
    outputs = _list_outputs(
        output_dir=output_dir, inputs=inputs, stdout=stdout, stderr=stderr
    )
    return outputs["out_dict"]


def reconstruction_callable(output_dir, inputs, stdout, stderr):
    outputs = _list_outputs(
        output_dir=output_dir, inputs=inputs, stdout=stdout, stderr=stderr
    )
    return outputs["reconstruction"]


def run_callable(output_dir, inputs, stdout, stderr):
    outputs = _list_outputs(
        output_dir=output_dir, inputs=inputs, stdout=stdout, stderr=stderr
    )
    return outputs["run"]


def session_callable(output_dir, inputs, stdout, stderr):
    outputs = _list_outputs(
        output_dir=output_dir, inputs=inputs, stdout=stdout, stderr=stderr
    )
    return outputs["session"]


def subject_callable(output_dir, inputs, stdout, stderr):
    outputs = _list_outputs(
        output_dir=output_dir, inputs=inputs, stdout=stdout, stderr=stderr
    )
    return outputs["subject"]


def suffix_callable(output_dir, inputs, stdout, stderr):
    outputs = _list_outputs(
        output_dir=output_dir, inputs=inputs, stdout=stdout, stderr=stderr
    )
    return outputs["suffix"]


def task_callable(output_dir, inputs, stdout, stderr):
    outputs = _list_outputs(
        output_dir=output_dir, inputs=inputs, stdout=stdout, stderr=stderr
    )
    return outputs["task"]


# Original source at L568 of <nipype-install>/interfaces/base/core.py
def _list_outputs(inputs=None, stdout=None, stderr=None, output_dir=None):
    return _results
